#=
Copyright (c) 2018-2022 Chris Coey, Lea Kapelevich, and contributors

This Julia package Hypatia.jl is released under the MIT license; see LICENSE
file in the root directory or at https://github.com/jump-dev/Hypatia.jl
=#

"""
Hypatia examples and script utilities.
"""
module Examples

using LinearAlgebra
# delete later, affects qr. see https://github.com/JuliaLang/julia/pull/40623
if VERSION < v"1.7.0-DEV.1188"
    const ColumnNorm = Val{true}
end

using Test
import Random
using Printf
import DataFrames
import CSV
import DataStructures: OrderedDict

import Hypatia
import Hypatia.PolyUtils
import Hypatia.Cones
import Hypatia.Models
import Hypatia.Solvers

include("model_utils.jl")
include("native_utils.jl")
include("JuMP_utils.jl")
include("spectral_functions_JuMP.jl")
include("benchmark_utils.jl")

const model_types = ["native", "JuMP"]

# list of names of native examples to run
const native_examples = [
    "densityest",
    "doptimaldesign",
    "linearopt",
    "matrixcompletion",
    "matrixregression",
    "maxvolume",
    "polyenvelope",
    "polymin",
    "portfolio",
    "sparsepca",
]

# list of names of JuMP examples to run
const JuMP_examples = [
    "CBLIB",
    "centralpolymat",
    "classicalquantum",
    "conditionnum",
    "contraction",
    "convexityparameter",
    "covarianceest",
    "densityest",
    "discretemaxlikelihood",
    "doptimaldesign",
    "entanglementassisted",
    "experimentdesign",
    "lotkavolterra",
    "lyapunovstability",
    "matrixcompletion",
    "matrixquadratic",
    "matrixregression",
    "maxvolume",
    "nearestcorrelation",
    "nearestpolymat",
    "nearestpsd",
    "nonparametricdistr",
    "normconepoly",
    "polyenvelope",
    "polymin",
    "polynorm",
    "portfolio",
    "regionofattr",
    "relentrentanglement",
    "robustgeomprog",
    "semidefinitepoly",
    "shapeconregr",
    "signomialmin",
    "sparselmi",
    "stabilitynumber",
]

load_example(mod::String, ex::String) = include(joinpath(@__DIR__, ex, mod * ".jl"))

function get_test_instances(mod::String, ex::String)
    return include(joinpath(@__DIR__, ex, mod * "_test.jl"))
end

function get_benchmark_instances(mod::String, ex::String)
    return include(joinpath(@__DIR__, ex, mod * "_benchmark.jl"))
end

model_type_examples(mod::String) = eval(Symbol(mod, "_examples"))

# load all examples
for mod in model_types, ex in model_type_examples(mod)
    load_example(mod, ex)
end

# build ordered dictionary of all test instances
function get_test_instances()
    test_insts = OrderedDict{String, OrderedDict}()
    for mod in model_types
        mod_insts = test_insts[mod] = OrderedDict()
        for ex in model_type_examples(mod)
            mod_insts[ex] = get_test_instances(mod, ex)
        end
    end
    return test_insts
end

end
